@@ -1,94 +1,24 @@
 package Music::Tag::MusicBrainz;
-our $VERSION = 0.30;
-
-# Copyright (c) 2006 Edward Allen III. Some rights reserved.
-#
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the Artistic License, distributed
-## with Perl.
-#
-
-=pod
-
-=head1 NAME
-
-Music::Tag::MusicBrainz - Plugin module for Music::Tag to get information from MusicBrainz database.
-
-=head1 SYNOPSIS
-
-	use Music::Tag
-
-	my $info = Music::Tag->new($filename);
-   
-	my $plugin = $info->add_plugin("MusicBrainz");
-	$plugin->get_tag;
-
-	print "Music Tag Track ID ", $info->mb_trackid();
-
-=head1 DESCRIPTION
-
-Music::Tag::MusicBrainz is normally created in Music::Tag. This plugin gathers additional information about a track from amazon, and updates the tag object.
-
-=head1 REQUIRED VALUES
-
-=over 4
-
-=item artist
-
-=back
-
-=head1 USED VALUES
-
-=over 4
-
-=item album
-
-This is used to filter results. 
-
-=item releasedate
-
-This is used to filter results. 
-
-=item totaltracks
-
-This is used to filter results. 
-
-=item title
-
-title is used only if track is not true, or if trust_title option is set.
-
-=item tracknum
-
-tracknum is used only if title is not true, or if trust_track option is set.
-
-=back
-
-=head1 SET VALUES
-
-=over 4
-
-=item album
-
-=item title
-
-title is set only if trust_track is true.
-
-=item track
-
-track is set only if track is not true or trust_title is true.
-
-=item releasedate
-
-
-=cut
-
 use strict;
 use warnings;
+our $VERSION = .40_01;
+
+# Copyright (c) 2006,2010 Edward Allen III. Some rights reserved.
+
+#
+# You may distribute under the terms of either the GNU General Public
+# License or the Artistic License, as specified in the README file.
+#
+
+
+
+
 use WebService::MusicBrainz::Artist;
 use WebService::MusicBrainz::Release;
 use WebService::MusicBrainz::Track;
+use Cache::FileCache;
 use utf8;
-our @ISA = qw(Music::Tag::Generic);
+use base qw(Music::Tag::Generic);
 
 sub default_options {
     {  prefered_country         => "US",
@@ -105,19 +35,13 @@ sub default_options {
     };
 }
 
-=back
+sub required_values {
+	return qw( artist)
+}
 
-=head1 METHODS
-
-=over 4
-
-=item get_tag
-
-Updates current tag object with information from MusicBrainz database.
-
-Same as $mbplugin->artist_info() && $mbplugin->album_info() && $mbplugin->track_info();
-
-=cut
+sub set_values {
+	return qw( album releasedate totaltracks title tracknum title track releasedate) 
+}
 
 sub get_tag {
     my $self = shift;
@@ -131,17 +55,11 @@ sub get_tag {
     return $self;
 }
 
-=item artist_info
-
-Update the tag object with information about the artist from MusicBrainz.
-
-=cut
-
 sub artist_info {
     my $self = shift;
     $self->status( "Looking up artist from " . $self->options->{mb_host} );
     unless ( exists $self->{mb_a} ) {
-        $self->{mb_a} = WebService::MusicBrainz::Artist->new( HOST => $self->options->{mb_host} );
+        $self->{mb_a} = WebService::MusicBrainz::Artist->new( HOST => $self->options->{mb_host}, CACHE => $self->mb_cache );
     }
     my $params   = {};
     my $maxscore = 0;
@@ -240,24 +158,21 @@ sub artist_info {
     return $self->info;
 }
 
-=item album_info
-
-Update the tag object with information about the album from MusicBrainz.
-
-=cut
-
 
 sub album_info {
     my $self = shift;
     $self->status( "Looking up album from " . $self->options->{mb_host} );
     unless ( exists $self->{mb_r} ) {
-        $self->{mb_r} = WebService::MusicBrainz::Release->new( HOST => $self->options->{mb_host} );
+        $self->{mb_r} = WebService::MusicBrainz::Release->new( HOST => $self->options->{mb_host}, CACHE => $self->mb_cache );
     }
     my $params = { LIMIT => 200 };
     my $release = undef;
     if (    ( defined $self->info->mb_albumid )
          && ( $self->info->mb_albumid )
-         && ( not $self->info->options->{ignore_mbid} ) ) {
+         && ( not $self->info->options->{ignore_mbid} ) 
+         && ( length($self->info->mb_albumid) > 30)
+         
+         ) {
         $params->{MBID} = $self->info->mb_albumid;
         my $response = $self->{mb_r}->search($params);
         $release = $response->release();
@@ -324,7 +239,7 @@ sub album_info {
                  and ( ( $self->info->totaltracks ) == ( $_->track_list->{count} ) ) ) {
                 $s += 16;
             }
-			if (     ( defined $self->info->disc)
+			if (     ( defined $self->info->disc) and (defined $disc)
 				  and ( ( $self->info->disc) == ( $disc) ) ) {
 				  $s += 8;
 			 }
@@ -406,12 +321,6 @@ sub album_info {
     return $self->info;
 }
 
-=item track_info
-
-Update the tag object with information about the track from MusicBrainz.
-
-=cut
-
 
 sub track_info {
     my $self = shift;
@@ -423,11 +332,11 @@ sub track_info {
     }
     $self->status( "Looking up track from " . $self->options->{mb_host} );
     unless ( exists $self->{mb_r} ) {
-        $self->{mb_r} = WebService::MusicBrainz::Release->new( HOST => $self->options->{mb_host} );
+        $self->{mb_r} = WebService::MusicBrainz::Release->new( HOST => $self->options->{mb_host}, CACHE => $self->mb_cache );
     }
     return unless ( defined $self->info->mb_albumid );
     my $params = { MBID => $self->info->mb_albumid,
-                   INC  => "tracks+puids+discs+release-events",
+                   INC  => "tracks discs release-events",
                  };
     my $response = $self->{mb_r}->search($params);
     unless ( $response->release->track_list ) {
@@ -484,7 +393,7 @@ sub track_info {
                 $s += 4;
             }
         }
-        if ( defined $self->info->duration ) {
+        if ((defined $self->info->duration) && (exists $t->{duration}) && (defined $t->{duration})) {
             my $diff = abs( $self->info->duration - $t->{duration} );
             if ( $diff < 3000 ) {
                 if ( $self->options->{trust_time} ) {
@@ -593,10 +502,144 @@ sub track_info {
     }
 }
 
-=item default_options
+sub mb_cache {
+    my $self = shift;
+	my $new = shift;
+	if ($new) {
+		$self->{mb_cache} = $new;
+	}
+    unless ( ( exists $self->{mb_cache} ) && ( $self->{mb_cache} ) ) {
+        if ( $self->options->{mb_cache} ) {
+            $self->{mb_cache} = $self->options->{mb_cache};
+        }
+        else {
+            $self->{mb_cache} =
+              Cache::FileCache->new(
+                                     { namespace          => "mb_cache",
+                                       default_expires_in => 60000,
+                                     }
+                                   );
+        }
+    }
+    return $self->{mb_cache};
+}
+
+
+1;
+__END__
+=pod
+
+=head1 NAME
+
+Music::Tag::MusicBrainz - Plugin module for Music::Tag to get information from MusicBrainz database.
+
+=head1 SYNOPSIS
+
+	use Music::Tag
+
+	my $info = Music::Tag->new($filename);
+   
+	my $plugin = $info->add_plugin("MusicBrainz");
+	$plugin->get_tag;
+
+	print "Music Tag Track ID ", $info->mb_trackid();
+
+=head1 DESCRIPTION
+
+This plugin gathers additional information about a track from L<www.musicbrianz.org>, and updates the Music::Tag object.
+
+Music::Tag::MusicBrainz objects must be created by Music::Tag.
+
+=head1 REQUIRED DATA VALUES
+
+=over 4
+
+=item artist
+
+=back
+
+=head1 USED DATA VALUES
+
+=over 4
+
+=item album
+
+This is used to filter results. 
+
+=item releasedate
+
+This is used to filter results. 
+
+=item totaltracks
+
+This is used to filter results. 
+
+=item title
+
+title is used only if track is not true, or if trust_title option is set.
+
+=item tracknum
+
+tracknum is used only if title is not true, or if trust_track option is set.
+
+=back
+
+=head1 SET DATA VALUES
+
+=over 4
+
+=item album
+
+=item title
+
+title is set only if trust_track is true.
+
+=item track
+
+track is set only if track is not true or trust_title is true.
+
+=item releasedate
+
+=back
+
+=head1 METHODS
+
+=over 4
+
+=item B<get_tag()>
+
+Updates current Music::Tag object with information from MusicBrainz database.
+
+Same as $mbplugin->artist_info() && $mbplugin->album_info() && $mbplugin->track_info();
+
+=item B<artist_info()>
+
+Update the Music::Tag object with information about the artist from MusicBrainz.
+
+=item B<album_info()>
+
+Update the Music::Tag object with information about the album from MusicBrainz.
+
+=item B<track_info()>
+
+Update the Music::Tag object with information about the track from MusicBrainz.
+
+=item B<mb_cache()>
+
+Returns and optionally sets a reference to the Cache::FileCache object used to cache requests.
+
+=item B<default_options()>
 
 Returns hash of default options for plugin
 
+=item B<required_values()>
+
+A list of required values required for get_tag() to work.
+
+=item B<set_values()>
+
+A list of values that can be set by this module.
+
 =back
 
 =head1 OPTIONS
@@ -653,15 +696,17 @@ Set to host for musicbrainz.  Default is www.musicbrainz.org.
 
 Sometimes will grab incorrect info. This is due to the lack of album level view when repairing tags.
 
-=head1 SEE ALSO INCLUDED
-
-L<Music::Tag>, L<Music::Tag::Amazon>, L<Music::Tag::File>, L<Music::Tag::FLAC>, L<Music::Tag::Lyrics>,
-L<Music::Tag::M4A>, L<Music::Tag::MP3>, L<Music::Tag::OGG>, L<Music::Tag::Option>
-
 =head1 SEE ALSO
 
-L<WebService::MusicBrainz>
+L<WebService::MusicBrainz>, L<Music::Tag>, L<www.musicbrianz.org>
 
+=head1 SOURCE
+
+Source is available at github: L<http://github.com/riemann42/Music-Tag-MusicBrainz|http://github.com/riemann42/Music-Tag-MusicBrainz>.
+
+=head1 BUG TRACKING
+
+Please use github for bug tracking: L<http://github.com/riemann42/Music-Tag-MusicBrainz/issues|http://github.com/riemann42/Music-Tag-MusicBrainz/issues>.
 
 =head1 AUTHOR 
 
@@ -670,14 +715,32 @@ Edward Allen III <ealleniii _at_ cpan _dot_ org>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007 Edward Allen III. Some rights reserved.
+Copyright (c) 2007,2008 Edward Allen III. Some rights reserved.
 
-This program is free software; you can redistribute it and/or
-modify it under the terms of the Artistic License, distributed
-with Perl.
+=head1 LICENSE
+
+This program is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either:
+
+a) the GNU General Public License as published by the Free
+Software Foundation; either version 1, or (at your option) any
+later version, or
+
+b) the "Artistic License" which comes with Perl.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
+the GNU General Public License or the Artistic License for more details.
+
+You should have received a copy of the Artistic License with this
+Kit, in the file named "Artistic".  If not, I'll be glad to provide one.
+
+You should also have received a copy of the GNU General Public License
+along with this program in the file named "Copying". If not, write to the
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA or visit their web page on the Internet at
+http://www.gnu.org/copyleft/gpl.html.
 
 
-=cut
 
-
-1;
